@@ -14,8 +14,10 @@
 #' 
 #' @family connection functions
 #' @return A opal object or a list of opal objects.
-#' @param username User name in opal(s). Can be provided by "opal.username" option.
+#' @param username User name in opal(s). Can be provided by "opal.username" option. 
 #' @param password User password in opal(s). Can be provided by "opal.password" option.
+#' @param token Personal access token (since opal 2.15). Only effective if the username or the password is NULL or empty. 
+#'  Can be provided by "opal.token" option.
 #' @param url Opal url or list of opal urls. Can be provided by "opal.url" option.
 #' @param opts Curl options as described by httr (call httr::httr_options() for details). Can be provided by "opal.opts" option.
 #' @param restore Workspace ID to be restored (see also opal.logout)
@@ -25,15 +27,25 @@
 #' #### The below examples illustrate the different ways to login in opal ####
 #'
 #' # explicite username/password login
-#' o <- opal.login(username='administrator',password='password',url='https://opal-demo.obiba.org')
+#' o <- opal.login(username='administrator', password='password', url='https://opal-demo.obiba.org')
 #' opal.logout(o)
 #'
-#' # login using options
+#'  # explicite personal access token login
+#' o <- opal.login(token='HYG16LO0VaX4O0UardNbiqmr2ByBpRke', url='https://opal-demo.obiba.org')
+#' opal.logout(o)
+#'
+#' # login using options and user credentials
 #' options(opal.username='administrator',
 #'  opal.password='password',
 #'  opal.url='https://opal-demo.obiba.org')
 #' o <- opal.login()
 #' opal.logout(o)
+#' 
+#' # login using options and personal access token
+#' options(opal.token='HYG16LO0VaX4O0UardNbiqmr2ByBpRke',
+#'  opal.url='https://opal-demo.obiba.org')
+#' o <- opal.login()
+#' opal.logout(o)
 #'
 #' # login using ssl key pair
 #' options(opal.opts=list(
@@ -42,12 +54,12 @@
 #' o <- opal.login(url='https://opal-demo.obiba.org')
 #' opal.logout(o)
 #'}
-opal.login <- function(username=getOption("opal.username"), password=getOption("opal.password"), url=getOption("opal.url"), opts=getOption("opal.opts", list()), restore=NULL) {
+opal.login <- function(username=getOption("opal.username"), password=getOption("opal.password"), token=getOption("opal.token"), url=getOption("opal.url"), opts=getOption("opal.opts", list()), restore=NULL) {
   if (is.null(url)) stop("opal url is required", call.=FALSE)
   if(is.list(url)){
-    lapply(url, function(u){opal.login(username, password, u, opts=opts, restore=restore)})
+    lapply(url, function(u){opal.login(username, password, token, u, opts=opts, restore=restore)})
   } else {
-    .opal.login(username, password, url, opts=opts, restore=restore)
+    .opal.login(username, password, token, url, opts=opts, restore=restore)
   }
 }
 
@@ -232,10 +244,17 @@ opal.delete <- function(opal, ..., query=list(), callback=NULL) {
 #' Constructs the value for the Authorization header
 #' @import jsonlite
 #' @keywords internal
-.authToken <- function(username, password) {
+.authorizationHeader <- function(username, password) {
   paste("X-Opal-Auth", jsonlite::base64_enc(paste(username, password, sep=":")))
 }
 
+#' Constructs the value for the X-Opal-Auth header
+#' @import jsonlite
+#' @keywords internal
+.tokenHeader <- function(token) {
+  token
+}
+
 #' Process response with default handler or the provided one
 #' @keywords internal
 .handleResponseOrCallback <- function(opal, response, callback=NULL) {
@@ -402,7 +421,7 @@ opal.delete <- function(opal, ..., query=list(), callback=NULL) {
 #' Create the opal object
 #' @import httr
 #' @keywords internal
-.opal.login <- function(username, password, url, opts=list(), restore=NULL) {
+.opal.login <- function(username, password, token, url, opts=list(), restore=NULL) {
   if (is.null(url)) stop("opal url is required", call.=FALSE)
   opal <- new.env(parent=globalenv())
   # Username
@@ -420,15 +439,29 @@ opal.delete <- function(opal, ..., query=list(), callback=NULL) {
     opts$encoding <- NULL # not a httr/curl option
   }
   
-  # authentication token
-  if(is.null(username) == FALSE) {
-    # Authorization
-    opal$authorization <- .authToken(username, password)
-  }
   # httr/curl options
   protocol <- strsplit(url, split="://")[[1]][1]
   options <- opts
-  if (protocol=="https") {
+  # legacy RCurl options to httr
+  if (!is.null(options$ssl.verifyhost)) {
+    options$ssl_verifyhost = options$ssl.verifyhost
+    options$ssl.verifyhost <- NULL
+  }
+  if (!is.null(options$ssl.verifypeer)) {
+    options$ssl_verifypeer = options$ssl.verifypeer
+    options$ssl.verifypeer <- NULL
+  }
+  
+  # authentication strategies
+  if(!is.na(username) && !is.null(username) && nchar(username) > 0 
+     && !is.na(password) && !is.null(password) && nchar(password) > 0) {
+    # Authorization header
+    opal$authorization <- .authorizationHeader(username, password)
+  } else if (!is.na(token) && !is.null(token) && nchar(token) > 0) {
+    # Token header
+    opal$token <- .tokenHeader(token)
+  } else if (protocol=="https") {
+    # Two-way SSL authentication
     if (!is.null(options$cainfo)) {
       options$cainfo <- .getPEMFilePath(options$cainfo)
     }
@@ -438,16 +471,10 @@ opal.delete <- function(opal, ..., query=list(), callback=NULL) {
     if (!is.null(options$sslkey)) {
       options$sslkey <- .getPEMFilePath(options$sslkey)
     }
-    # legacy RCurl options to httr
-    if (!is.null(options$ssl.verifyhost)) {
-      options$ssl_verifyhost = options$ssl.verifyhost
-      options$ssl.verifyhost <- NULL
-    }
-    if (!is.null(options$ssl.verifypeer)) {
-      options$ssl_verifypeer = options$ssl.verifypeer
-      options$ssl.verifypeer <- NULL
-    }
+  } else {
+    stop("opal authentication strategy not identified: either provide username/password or API access token or SSL certificate/private keys", call.=FALSE)
   }
+  
   opal$config <- config()
   opal$config$options <- options
   opal$handle <- handle(paste0(opal$url, "/", sample(1000:9999, 1))) # append a random number to ensure urls are different
@@ -456,11 +483,9 @@ opal.delete <- function(opal, ..., query=list(), callback=NULL) {
   class(opal) <- "opal"
   
   # get user profile to test sign-in
-  r <- GET(.url(opal, "system", "subject-profile", "_current"), config = opal$config, httr::add_headers(Authorization = opal$authorization), handle = opal$handle, .verbose())
+  r <- GET(.url(opal, "system", "subject-profile", "_current"), config = opal$config, httr::add_headers(Authorization = opal$authorization, 'X-Opal-Auth' = opal$token), handle = opal$handle, .verbose())
   opal$profile <- .handleResponse(opal, r)
-  if(is.null(username)) {
-    opal$username <- opal$profile$principal
-  }
+  opal$username <- opal$profile$principal
   
   opal
 }
